package org.apache.turbine.exception;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Turbine" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Turbine", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

import org.apache.turbine.ExceptionHandler;
import org.apache.turbine.RunData;
import org.apache.turbine.Turbine;
import org.apache.turbine.ValveContext;
import org.apache.turbine.pipeline.DefaultTargetValve;
import org.apache.turbine.pipeline.CleanUpValve;
import org.apache.commons.lang.exception.ExceptionUtils;

/**
 * The default exception handler for {@link Turbine}. Populates the RunData
 * with information on the throwable, and then renders the template specified
 * by the 'template.error' configuration property.
 *
 * @author <a href="mailto:james@jamestaylor.org">James Taylor</a>
 * @version $Id: DefaultExceptionHandler.java,v 1.7 2003/11/23 00:42:22 mpoeschl Exp $
 */
public class DefaultExceptionHandler implements ExceptionHandler
{
    /**
     * @see ExceptionHandler#handleException
     */
    public void handleException( RunData data, Throwable throwable )
        throws Exception
    {
        data.setTarget( Turbine.getConfiguration()
            .getString( "template.error", "/Error.vm" ) );

        data.setStackTrace( ExceptionUtils.getStackTrace( throwable ), throwable );

        try 
        {
            // Simulate a minimal Pipeline here to run the target.
            new DefaultTargetValve().invoke(data, new NullValveContext());
        }
        finally
        {
            // generally pull tools will need to be cleaned up.
            new CleanUpValve().invoke(data, new NullValveContext());
        }
    }

    /**
     * A skeleton {@link org.apache.turbine.ValveContext}
     * implementation to assist in invoking DefaultTargetValve in
     * {@link #handleException(RunData, Throwable)}.
     */
    protected final class NullValveContext implements ValveContext
    {
        /**
         * No-op implementation of ValveContext interface.
         */
        public void invokeNext(RunData ignored)
        {
            // Do nothing
        }
    }
}
