package org.apache.turbine.pipeline;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Turbine" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Turbine", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

import org.apache.turbine.Turbine;
import org.apache.turbine.RunData;
import org.apache.turbine.modules.Module;
import org.apache.turbine.TurbineException;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;
/**
 * This class is used execute the module chain.
 *
 * @author <a href="mailto:mikeh@apache.org">Mike Haberman</a>
 * @version $Id: ModuleRunner.java,v 1.4 2002/04/09 17:29:35 jtaylor Exp $
 */
public class ModuleRunner
{
    private static final Log log = LogFactory.getLog( ModuleRunner.class );

    public void run(String moduleType, RunData data)
        throws TurbineException,Exception
    {
        String target; 
        String oldTarget = "";
        Module module; 

        target = getTarget(data); 
        while(! oldTarget.equals(target)) 
        {
           log.debug("Module to run: " + moduleType + ":" + target);

           module = Turbine.getResolver().getModule(moduleType, target);
           module.execute(data);

           // target may have changed
           oldTarget = target;
           target = getTarget(data);
        }

        /*
        // set the run data's target to the normalized target path
        // i.e. ',' are changed to '/'
        NOT A good idea.  Template Services expect the ','
        data.setTarget(target);
        */
    }

    private String getTarget(RunData data)
        throws Exception
    {
        String target = data.getTarget();
        if (target == null) 
        {
            target = data.getParameters().getString("template");
            data.setTarget(target);
        }

        // normalize the target
        target = data.getTarget();
        StringBuffer sb = new StringBuffer();
        PipelineUtil.parseTemplatePath(target,sb);
        return sb.toString();
    }
}
