package org.apache.turbine;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Turbine" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Turbine", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

import java.util.HashMap;
import java.util.Map;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * Tests the Turbine servlet.
 *
 * @author <a href="mailto:dlr@finemaltcoding.com">Daniel Rall</a>
 * @since 3.0
 * @version $Id: TurbineTest.java,v 1.2 2002/04/17 17:22:31 dlr Exp $
 */
public class TurbineTest
    extends TestCase
    implements TurbineConstants
{
    /**
     * Creates a new instance.
     */
    public TurbineTest(String testName) 
    {
        super(testName);
    }

    /**
     * Return the Test
     */
    public static Test suite() 
    {
        return new TestSuite(TurbineTest.class);
    }

    /**
     * Setup the test.
     */
    public void setUp() 
    {
        // Nothing done here yet.
    }
   
    /**
     * Tear down the test.
     */
    public void tearDown() 
    {
        // Nothing to do here yet.
    }

    /**
     * Tests the servlet configuration and configuration extraction.
     */
    public void testServletConfig()
    {
        try
        {
            Map initParams = new HashMap(1);
            TurbineConfig config = new TurbineConfig(".", initParams);

            // Test of TurbineConfig::setAttribute()/getAttribute()
            config.setAttribute("foo", "bar");
            assertEquals("bar", (String) config.getAttribute("foo"));

            // Test of Turbine::findParameter()
            String testRoot = "/test";
            String expectedLoggingRoot = "/initParams/logs";
            initParams.put(LOGGING_ROOT, expectedLoggingRoot);
            String loggingRoot = Turbine.findInitParameter(config, config,
                                                           LOGGING_ROOT,
                                                           testRoot);
            assertNotNull("Turbine.findInitParameter() returned null when " +
                          "supplied with a non-null default", loggingRoot);
            assertEquals(expectedLoggingRoot, loggingRoot);
            initParams.remove(LOGGING_ROOT);
            // A discrepancy between the Map key and the requested
            // parameter is used to test the ability to always check
            // for short parameters names inside the Turbine
            // namespace.
            String loggingRootKey = CONFIG_NAMESPACE + '.' + LOGGING_ROOT;
            initParams.put(loggingRootKey, expectedLoggingRoot);
            loggingRoot = Turbine.findInitParameter(config, config,
                                                    LOGGING_ROOT, testRoot);
            assertEquals(expectedLoggingRoot, loggingRoot);
            initParams.remove(loggingRootKey);
            loggingRoot = Turbine.findInitParameter(config, config,
                                                    LOGGING_ROOT, testRoot);
            assertEquals("Turbine.findInitParameter() returned an incorrect " +
                         "default value", "/test", loggingRoot);
        }            
        catch (Exception e)
        {
            e.printStackTrace();
            fail(e.getMessage());
        }
    }
}
