package org.apache.turbine.pipeline;

/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2001-2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" and
 *    "Apache Turbine" must not be used to endorse or promote products
 *    derived from this software without prior written permission. For
 *    written permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    "Apache Turbine", nor may "Apache" appear in their name, without
 *    prior written permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */


import junit.framework.TestCase;

import org.apache.turbine.Pipeline;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;

/**
 * Tests TurbinePipeline.
 *
 * @author <a href="mailto:dlr@finemaltcoding.com">Daniel Rall</a>
 * @version $Id: PipelineCreationTest.java,v 1.2 2004/11/29 10:11:36 epugh Exp $
 */
public class PipelineCreationTest extends TestCase
{
    private Pipeline pipeline;
    /**
     * Constructor
     */
    public PipelineCreationTest(String testName)
    {
        super(testName);
    }
    
    public void setUp(){
        pipeline = new TurbinePipeline();
        pipeline.addValve(new SimpleValve());
        pipeline.addValve(new DetermineActionValve());
		TargetExtensionConditionalValve cValve = new TargetExtensionConditionalValve();
		cValve.setExtension(".vm");
		pipeline.addValve(cValve);
    }

    
    public void testSavingPipelineWXstream() throws Exception
    {
        XStream xstream = new XStream(new DomDriver()); // does not require XPP3 library
       
        String xml = xstream.toXML(pipeline);
        // System.out.println(xml);
        //Pipeline pipeline = (Pipeline)xstream.fromXML(xml);
        
    }
    
    public void testReadingPipelineWXstream() throws Exception{
        String xml="<org.apache.turbine.pipeline.TurbinePipeline>  <valves>    <org.apache.turbine.pipeline.SimpleValve/>    <org.apache.turbine.pipeline.DetermineActionValve/>"
		            + "<org.apache.turbine.pipeline.TargetExtensionConditionalValve> <extension>.vm</extension>"
					+ "<pipelines>"
					+ "<org.apache.turbine.pipeline.TurbinePipeline> <valves> <org.apache.turbine.pipeline.SimpleValve/> </valves> </org.apache.turbine.pipeline.TurbinePipeline>"
					+ "</pipelines>"					
					+ "</org.apache.turbine.pipeline.TargetExtensionConditionalValve>"
		            + "<org.apache.turbine.pipeline.TargetExtensionConditionalValve> <extension>.xml</extension>"
					+ "<pipelines>"
					+ "<org.apache.turbine.pipeline.TurbinePipeline> <valves> <org.apache.turbine.pipeline.SimpleValve/> </valves> </org.apache.turbine.pipeline.TurbinePipeline>"
					+ "</pipelines>"
					+ "</org.apache.turbine.pipeline.TargetExtensionConditionalValve>"
					+ " </valves></org.apache.turbine.pipeline.TurbinePipeline>";
        XStream xstream = new XStream(new DomDriver()); // does not require XPP3 library
        Object o = xstream.fromXML(xml);
        Pipeline pipeline = (Pipeline)o;
        assertEquals(pipeline.getValves().length,4);
        assertTrue(pipeline.getValves()[0] instanceof SimpleValve);
        assertTrue(pipeline.getValves()[1] instanceof DetermineActionValve);
		assertTrue(pipeline.getValves()[2] instanceof TargetExtensionConditionalValve);
		assertTrue(pipeline.getValves()[3] instanceof TargetExtensionConditionalValve);
		
        // String xmlString = xstream.toXML(pipeline);
        // System.out.println(xmlString);		
    }

}
